@@ -103,56 +103,6 @@ pub(crate) enum Event {
     },
 }
 
-impl Event {
-    pub(crate) fn with_new_actor(self, idx: usize) -> Self {
-        match self {
-            Self::PutMap {
-                key,
-                value,
-                id,
-                conflict,
-            } => Self::PutMap {
-                key,
-                value,
-                id: id.with_new_actor(idx),
-                conflict,
-            },
-            Self::PutSeq {
-                index,
-                value,
-                id,
-                conflict,
-            } => Self::PutSeq {
-                index,
-                value,
-                id: id.with_new_actor(idx),
-                conflict,
-            },
-            Self::Insert {
-                index,
-                value,
-                id,
-                conflict,
-            } => Self::Insert {
-                index,
-                value,
-                id: id.with_new_actor(idx),
-                conflict,
-            },
-            Self::IncrementMap { key, n, id } => Self::IncrementMap {
-                key,
-                n,
-                id: id.with_new_actor(idx),
-            },
-            Self::IncrementSeq { index, n, id } => Self::IncrementSeq {
-                index,
-                n,
-                id: id.with_new_actor(idx),
-            },
-            event => event,
-        }
-    }
-}
 
 impl PatchLog {
     /// Create a new [`PatchLog`]
@@ -438,11 +388,19 @@ impl PatchLog {
     }
 
     pub(crate) fn migrate_actor(&mut self, index: usize) {
-        let dirty = std::mem::take(&mut self.events);
-        self.events = dirty
-            .into_iter()
-            .map(|(o, e)| (o.with_new_actor(index), e.with_new_actor(index)))
-            .collect();
+        for (o, e) in &mut self.events {
+            *o = o.with_new_actor(index);
+            match e {
+                Event::PutMap { id, .. }
+                | Event::PutSeq { id, .. }
+                | Event::Insert { id, .. }
+                | Event::IncrementMap { id, .. }
+                | Event::IncrementSeq { id, .. } => {
+                    *id = id.with_new_actor(index);
+                }
+                _ => {}
+            }
+        }
 
         let dirty = std::mem::take(&mut self.expose);
         self.expose = dirty
@@ -455,26 +413,23 @@ impl PatchLog {
     // this is an uncommon operation so this seems preferable to storing ExId's in place
     // for every objid and opid
     pub(crate) fn migrate_actors(&mut self, others: &Vec<ActorId>) -> Result<(), AutomergeError> {
-        if &self.actors == others {
-            return Ok(());
-        }
-        if self.actors.is_empty() {
-            self.actors = others.clone();
-            return Ok(());
-        }
-        for i in 0..others.len() {
-            let b = &others[i];
-            if let Some(a) = self.actors.get(i) {
-                if a == b {
-                    // Same actor at position i
-                } else if b < a {
-                    self.actors.insert(i, b.clone());
-                    self.migrate_actor(i);
-                } else {
-                    return Err(AutomergeError::PatchLogMismatch);
+        if &self.actors != others {
+            if self.actors.is_empty() {
+                self.actors = others.clone();
+                return Ok(());
+            }
+            for i in 0..others.len() {
+                match (self.actors.get(i), others.get(i)) {
+                    (Some(a), Some(b)) if a == b => {}
+                    (Some(a), Some(b)) if b < a => {
+                        self.actors.insert(i, b.clone());
+                        self.migrate_actor(i);
+                    }
+                    (None, Some(b)) => {
+                        self.actors.insert(i, b.clone());
+                    }
+                    _ => return Err(AutomergeError::PatchLogMismatch),
                 }
-            } else {
-                self.actors.insert(i, b.clone());
             }
         }
         Ok(())
